/***************************************************************************
 *   Copyright (C) 2014-2021 by Tomasz Bojczuk                             *
 *   seelook@gmail.com                                                     *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 3 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *  You should have received a copy of the GNU General Public License      *
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.  *
 ***************************************************************************/

#include "tmeasure.h"
#include "tchunk.h"

Tmeasure::Tmeasure(int nr, Tmeter::Emeter m)
    : m_number(nr)
    , m_meter(m)
    , m_duration(0)
{
}

void Tmeasure::addNote(const Tchunk &n)
{
    m_notes << n;
    if (m_meter.meter() != Tmeter::NoMeter)
        m_duration += n.duration();
}

void Tmeasure::removeLastNote()
{
    m_duration -= lastNote().duration();
    if (m_meter.meter() != Tmeter::NoMeter)
        m_notes.removeLast();
}

bool Tmeasure::isFull()
{
    return m_duration == m_meter.duration();
}

void Tmeasure::swapWithNotes(int noteNr, const QList<Tchunk> &notes)
{
    m_notes.replace(noteNr, notes.first());
    for (int n = 1; n < notes.count(); ++n)
        m_notes.insert(noteNr + n, notes[n]);
}

// #################################################################################################
// ###################                PROTECTED         ############################################
// #################################################################################################

void Tmeasure::prepend(const Tchunk &n)
{
    m_notes.prepend(n);
    if (m_meter.meter() != Tmeter::NoMeter)
        m_duration += n.duration();
    // In fact, anacrusis measures have no sense in when no meter, but... let's have this check
}
